%System used to Compute Lyapunov Exponents for continuous
%  n by n systems via QR factorization
%
%       xdot = f(x)            continuous dynamical system
%       Qdot = QR            QR Factorization
%       rhodot = Q'JQ       
%
%       where the lyapunov exponents are lim t->inf (rhp(i,i)/t)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% t : time array
% IC = [x0 y1 y2 y3 p] is an (nxn + 2n) column vector of initial conditions
%  x0: n by 1 column vector of initial conditions for the original system
% y_i: n by 1 orthonormal basis. 
%   p: n by 1 column vector of initial Lyapunov exponents [0 0 0]';


function dXdt = LyapunovSys_QR(t, IC, sysName, LinearSysName, param)

%determine the dimension of the system n^2 +2n -L = 0
L = length(IC);
n = (-2 + sqrt(4 + 4*L))/2;

% Intial condition of system
x0 = IC(1:n);

% Initial Orthonormal Basis. Reshape into an nxn matrix
Q = reshape(IC(n+1:end-n), n, n);

%Compute Jacobian of the Linear system about x0
J = feval(LinearSysName, x0, param);

% Perform 'economy' QR factorization
%[Q R] = qr(Y,0);

% Define P =  Q'JQ 
P = Q'*J*Q;

% Compute S = Q'*Qdot
SLow = P;                          %S upper triangular
SUp = -P';                       %S lower triangular
S = tril(SLow, -1) + triu(SUp, 1);

% SYSTEM That Solves Lyapunov Exponents 
% Lyapunov exponents = lim t->inf (rho(i,i)/t)
dy = feval(sysName, t, x0, param);
dQ = Q*S;
drho = diag(P);

dXdt = [dy; reshape(dQ, n^2, 1); drho];
